{
=============================================================================
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  |
| AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    |
| IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   |
| ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR  |
| ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL       |
| DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR   |
| SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER   |
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT           |
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY    |
| OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  |
| DAMAGE.                                                                      |
=============================================================================
   Author:  D.J.C. Blijker.
   Date:    24-4-2003
   version: 2.0
   e-mail:  c.blijker@wxs.nl
=============================================================================
  This unit will work in Delphi as well as in Kylix!!!
=============================================================================

  Project: Log-Daemon
  Module: LogDaemon_LogFile
=============================================================================
}
unit LogDaemon_LogFileLayer;

interface

uses
  SysUtils, IniFiles, Forms, extctrls;

type

  TDaemonLogFile = class
  private
    bFileIsOpen: boolean;
    iCurrentNr: integer;
    iMaxNr: integer;
    dTimeBetweenSwitching: TDateTime;
    dStartTime: TDateTime;
    sFileName: string;
    sExtension: string;
    F: TextFile;
    function  GetSwitchTime: integer;
    procedure SetSwitchTime( iTimeInMin: integer );
    procedure SetMaxVersionNr( iNr: integer );
    procedure SetVersionNr(const Value: integer);
    procedure CheckFile;
    procedure ReadIniFile;
    procedure WriteVersionAndTimeToIni;
    procedure SetFileOpen(const Value: boolean);
    procedure SetFileName( sBaseFileName: string );
  public
    constructor Create( sBaseFileName, sFileExtension: string ); overload;
    constructor Create( sBaseFileName: string ); overload;
    destructor Destroy; override;
    procedure WriteFormated( sFormat: string; const Args: array of const );
    procedure WriteToFile( sLine: string );
    property FileOpen: boolean read bFileIsOpen write SetFileOpen;
    property SwitchTimeInMinutes: integer read GetSwitchTime write SetSwitchTime;
    property CurrentVersionNr: integer read iCurrentNr write SetVersionNr;
    property MaxVersionNr: integer read iMaxNr write SetMaxVersionNr;
    property FilePathName: string read sFileName write SetFileName;
  end;

implementation

constructor TDaemonLogFile.Create( sBaseFileName: string );
begin
  Create( sBaseFileName, 'log' );
end;

constructor TDaemonLogFile.Create( sBaseFileName, sFileExtension: string );
begin
  dTimeBetweenSwitching := 1/24; // default every hour

// Check whether a pathname is supplied.
  if ExtractFilePath( sBaseFileName ) = '' then
    sFileName := ExtractFilePath( application.exename )+sBaseFileName
  else
    sFileName := sBaseFileName;

  if Pos('.', sFileExtension ) = 0 then
    sExtension := '.'+sFileExtension
  else
    sExtension := sFileExtension;

  iMaxNr := 10; // Untill told otherwise

  ReadIniFile; // From local exe-directory
end;

destructor TDaemonLogFile.Destroy;
begin
  if bFileIsOpen then
    CloseFile( F );

  WriteVersionAndTimeToIni;

  inherited;
end;

procedure TDaemonLogFile.WriteVersionAndTimeToIni;
var
  ini: TIniFile;
  iDay, iMonth, iYear, iHr, iMin, iSec, iMsec: word;
begin
  DecodeDate( dStartTime, iYear, iMonth, iDay );
  DecodeTime( dStartTime, iHr, iMin, iSec, iMsec );

  ini := TIniFile.Create( ExtractFilePath( application.exename )+
                          ExtractFileName( sFileName )+'.ini' );
  try
    ini.WriteInteger('DATA', 'version', iCurrentNr );
    ini.WriteInteger('DATA', 'switch_time', SwitchTimeInMinutes );
    ini.WriteInteger('DATA', 'max_version', iMaxNr );
    ini.WriteInteger('DATA', 'day', iDay );
    ini.WriteInteger('DATA', 'month', iMonth );
    ini.WriteInteger('DATA', 'year', iYear );
    ini.WriteInteger('DATA', 'hour', iHr );
    ini.WriteInteger('DATA', 'min', iMin );
    ini.WriteInteger('DATA', 'sec', iSec );
  finally
    ini.Free;
  end;
end;

procedure TDaemonLogFile.ReadIniFile;
var
  ini: TIniFile;
  iDay, iMonth, iYear, iHr, iMin, iSec, iST: word;
begin
  ini := TIniFile.Create( ExtractFilePath( application.exename )+
                          ExtractFileName( sFileName )+'.ini' );
  try
    iCurrentNr := ini.ReadInteger('DATA', 'version', 1 );
    iST := ini.ReadInteger('DATA', 'switch_time', 60 );
    iMaxNr := ini.ReadInteger('DATA', 'max_version', 10 );
    dTimeBetweenSwitching := iST /( 24 * 60 );
    iDay := ini.ReadInteger('DATA', 'day', 0 );
    iMonth := ini.ReadInteger('DATA', 'month', 0 );
    iYear := ini.ReadInteger('DATA', 'year', 0 );
    iHr := ini.ReadInteger('DATA', 'hour', 99 );
    iMin := ini.ReadInteger('DATA', 'min', 99 );
    iSec := ini.ReadInteger('DATA', 'sec', 99 );

    if( iDay = 0 )or( iMonth = 0 )or( iYear = 0 )or
    ( iHr = 99 )or( iMin = 99 )or( iSec = 99 )then
      dStartTime := Now
    else
    begin
      dStartTime := EncodeDate( iYear, iMonth, iDay );
      dStartTime := dStartTime + EncodeTime( iHr, iMin, iSec, 0 );
    end;
  finally
    ini.Free;
  end;
end;

function TDaemonLogFile.GetSwitchTime: integer;
begin
  result := Round( dTimeBetweenSwitching * 24 * 60 );
end;

procedure TDaemonLogFile.SetMaxVersionNr(iNr: integer);
begin
  if iNr > 0 then
  begin
    iMaxNr := iNr;
    WriteVersionAndTimeToIni;
  end;
end;

procedure TDaemonLogFile.SetVersionNr(const Value: integer);
begin
  if Value > 0 then
  begin
    iCurrentNr := Value;
    WriteVersionAndTimeToIni;
  end;
end;

procedure TDaemonLogFile.SetSwitchTime( iTimeInMin: integer );
begin
  dTimeBetweenSwitching := Abs( iTimeInMin / ( 24 * 60 ));
  WriteVersionAndTimeToIni;
end;

procedure TDaemonLogFile.CheckFile;
var
  dNow: TDateTime;
  sFile: string;
begin
  if dTimeBetweenSwitching <= 0 then
    exit;  // DON'T switch..

  dNow := Now;
  if dNow - dStartTime >= dTimeBetweenSwitching then
  begin
    dStartTime := dNow;
    Inc( iCurrentNr );
    if iCurrentNr > iMaxNr then
      iCurrentNr := 1;

    sFile := sFileName + '_' + IntToStr( iCurrentNr ) + sExtension;

    DeleteFile( sFile );

    WriteVersionAndTimeToIni;
  end;
end;

procedure TDaemonLogFile.WriteToFile( sLine: string );
begin
  WriteLn( F, sLine );
end;

procedure TDaemonLogFile.WriteFormated( sFormat: string;
                                        const Args: array of const );
var
  s: string;
begin
  s := Format( sFormat, Args );
  WriteToFile( s );
end;

procedure TDaemonLogFile.SetFileOpen(const Value: boolean);
var
  sFile: string;
begin
  if Value then
  begin
    CheckFile;
    sFile := sFileName+'_'+IntToStr( iCurrentNr )+ sExtension;

    AssignFile( F, sFile );
    if FileExists( sFile ) then
      Append( F )
    else
      Rewrite( F );

    bFileIsOpen := True;
  end
  else
  begin
    CloseFile( F );
    bFileIsOpen := False;
  end;
end;

procedure TDaemonLogFile.SetFileName( sBaseFileName: string );
var
  bFileWasOpen: boolean;
begin
  if bFileIsOpen then
  begin
    CloseFile( F );
    bFileIsOpen := False;
    bFileWasOpen := True;
  end
  else
    bFileWasOpen := False;

// Check whether there is an extension supplied..
  sExtension := ExtractFileExt( sBaseFileName );
  if sExtension = '' then
    sExtension := '.log';

// Check whether a path name is supplied..
  if ExtractFilePath( sBaseFileName ) = '' then
    sFileName := ExtractFilePath( application.exename )+sBaseFileName
  else
    sFileName := sBaseFileName;

  CurrentVersionNr := 1; // Start with new serial number..
  WriteVersionAndTimeToIni;

  if bFileWasOpen then
    FileOpen := True;
end;

end.
